/***************************************************************************
                          basicwidget.cpp  -  description
                             -------------------
    begin                : Don Nov 14 2002
    copyright            : (C) 2002 by Matthias Reif
    email                : matthias.reif@informatik.tu-chemnitz.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "basicwidget.h"

BasicWidget::BasicWidget(WCard *pCard, Options *pOptions, QWidget *parent, const char *name ) : QWidget(parent,name)
{

  card = pCard;
  options = pOptions;

  notString = new QString("<" + i18n ("not a wlan-device") + ">");

  deviceLabel = new QLabel(this);
  deviceLabel->setText(i18n ("Device") + ":");
  deviceLabel->move(STARTX, STARTY + STEPY*0);
  deviceLabel->resize(100, 20);  
  deviceLabel->show();

  deviceEdit = new KLineEdit(this);
  deviceEdit->setText(card->getDevice());
  deviceEdit->move(STARTX2, STARTY + STEPY*0);
  deviceEdit->resize(100, 20);
  deviceEdit->show();

  IPLabel1 = new QLabel(this);
  IPLabel1->setText(i18n ("IP-Address") + ":");
  IPLabel1->resize(100, 20);
  IPLabel1->move(STARTX, STARTY + STEPY*1);
  IPLabel1->show();

  IPLabel2 = new QLabel(this);
  IPLabel2->setText(card->getIP());
  IPLabel2->resize(100, 20);
  IPLabel2->move(STARTX2, STARTY + STEPY*1);
  IPLabel2->show();

  modeLabel = new QLabel(this);
  modeLabel->setText(i18n ("Mode") + ":");
  modeLabel->resize(150, 20);  
  modeLabel->move(STARTX, STARTY + STEPY*2);
  modeLabel->show();

  modeCombo = new KComboBox(this);
  modeCombo->resize(100, 20);
  modeCombo->move(STARTX2, STARTY + STEPY*2);
  modeCombo->insertItem("Auto" ,      IW_MODE_AUTO);
  modeCombo->insertItem("Ad-hoc" ,    IW_MODE_ADHOC);
  modeCombo->insertItem("Infrastr." , IW_MODE_INFRA);
  modeCombo->insertItem("Master" ,    IW_MODE_MASTER);
  modeCombo->insertItem("Repeater" ,  IW_MODE_REPEAT);
  modeCombo->insertItem("Second" ,    IW_MODE_SECOND);
  modeCombo->show();

  modeErrLabel = new QLabel(this);
  modeErrLabel->setText(*notString);
  modeErrLabel->resize(150, 20);
  modeErrLabel->move(modeCombo->pos());
  modeErrLabel->hide();
                                                          
  
  linkLabel = new QLabel(this);
  linkLabel->setText(i18n ("Link Quality") + ":");
  linkLabel->resize(100, 20);
  linkLabel->move(STARTX, STARTY + STEPY*3);
  linkLabel->show();

  linkBar = new KProgress(this);
  linkBar->resize(100, 20);
  linkBar->move(STARTX2, STARTY + STEPY*3);
  linkBar->show();

  linkErrLabel = new QLabel(this);
  linkErrLabel->setText(*notString);
  linkErrLabel->resize(150, 20);
  linkErrLabel->move(linkBar->pos());
  linkErrLabel->hide();

  avgLabel = new QLabel(this);
  avgLabel->setText(i18n ("Average L.Q.") + ":");
  avgLabel->resize(100, 20);
  avgLabel->move(STARTX, STARTY + STEPY*4);
  avgLabel->hide();

  avgBar = new KProgress(this);
  avgBar->resize(100, 20);
  avgBar->move(STARTX2, STARTY + STEPY*4);
  avgBar->hide();

  avgErrLabel = new QLabel(this);
  avgErrLabel->setText(*notString);
  avgErrLabel->resize(150, 20);
  avgErrLabel->move(avgBar->pos());
  avgErrLabel->hide();

  signalLabel1 = new QLabel(this);
  signalLabel1->setText(i18n ("Signal Level") + ":");
  signalLabel1->resize(100, 20);
  signalLabel1->move(STARTX, STARTY + STEPY*4);
  signalLabel1->show();

  signalBar = new KProgress(this);
  signalBar->resize(100, 20);
  signalBar->move(STARTX2, STARTY + STEPY*4);
  signalBar->show();

  signalLabel2 = new QLabel(this);
  signalLabel2->resize(150, 20);
  signalLabel2->move(signalBar->pos());
  signalLabel2->hide();

  noiseLabel1 = new QLabel(this);
  noiseLabel1->setText(i18n ("Noise Level") + ":");
  noiseLabel1->resize(100, 20);
  noiseLabel1->move(STARTX, STARTY + STEPY*5);
  noiseLabel1->show();

  noiseBar = new KProgress(this);
  noiseBar->resize(100, 20);
  noiseBar->move(STARTX2, STARTY + STEPY*5);
  noiseBar->show();

  noiseLabel2 = new QLabel(this);
  noiseLabel2->resize(150, 20);
  noiseLabel2->move(noiseBar->pos());
  noiseLabel2->hide();
  

  encodeLabel1 = new QLabel(this);
  encodeLabel1->setText(i18n ("Encoding") + ":");
  encodeLabel1->resize(100, 20);
  encodeLabel1->move(STARTX, STARTY + STEPY*6);
  encodeLabel1->show();

  encodeLabel2 = new QLabel(this);
  encodeLabel2->resize(150, 20);
  encodeLabel2->move(STARTX2, STARTY + STEPY*6);
  encodeLabel2->show();

  pollingLabel1 = new QLabel(this);
  pollingLabel1->setText(i18n ("Polling") + ":");
  pollingLabel1->resize(100, 20);
  pollingLabel1->move(STARTX, STARTY + STEPY*7);
  pollingLabel1->show();

  pollingSpin = new PollingSpinBox(1, 100, 1, this);
  pollingSpin->setValue(options->getPolling());
  pollingSpin->resize(50, 20);
  pollingSpin->move(STARTX2, STARTY + STEPY*7);
  pollingSpin->show();

  pollingLabel2 = new QLabel(this);
  pollingLabel2->setText(i18n ("seconds"));
  pollingLabel2->resize(100, 20);
  pollingLabel2->move(STARTX2+55, STARTY + STEPY*7);
  pollingLabel2->show();

 
  connect(deviceEdit, SIGNAL(returnPressed()), this, SLOT(slotApply()));
  connect(modeCombo, SIGNAL(activated(int)), this, SLOT(slotMode(int)));

  connect(pollingSpin, SIGNAL(valueChanged(int)), options, SLOT(slotPolling(int)));

  slotApply();
}

BasicWidget::~BasicWidget(){
}

/** No descriptions */
void BasicWidget::slotApply()
{
  card->setDevice(deviceEdit->text().data());
  IPLabel2->setText(card->getIP());
  if(card->isWave())
  {
    linkErrLabel->hide();
    avgErrLabel->hide();
    linkBar->show();
    avgBar->show();
    modeErrLabel->hide();
    modeCombo->show();
    modeCombo->setCurrentItem(card->getMode());

    char* key = (char*)malloc(IW_ENCODING_TOKEN_MAX);
    
    if(card->getEncode(key))
      encodeLabel2->setText(key);
    else
      encodeLabel2->setText("<" + i18n ("disabled") + ">");
             
  }else{
    linkBar->hide();
    avgBar->hide();
    signalBar->hide();
    noiseBar->hide();
    linkErrLabel->show();
    avgErrLabel->show();
    signalLabel2->show();
    noiseLabel2->show();
            
    modeCombo->hide();
    modeErrLabel->show();

    signalLabel2->setText(*notString);
    noiseLabel2->setText(*notString);
    encodeLabel2->setText(*notString);
  }
  
}

/** No descriptions */
void BasicWidget::slotMode(int index)
{
  int newMode = card->setMode(index);
  modeCombo->setCurrentItem(newMode);
}

/** No descriptions */
void BasicWidget::setLink(int link, int avg, int signal, int noise)
{
  if(card->isWave())
  {
    linkBar->setProgress(link);
    avgBar->setProgress(avg);
    if( link == 0 )
    {
      signalLabel2->hide();
      signalBar->show();
      signalBar->setProgress(signal);

      noiseLabel2->hide();
      noiseBar->show();
      noiseBar->setProgress(noise);
    }else{

      signalBar->hide();
      signalLabel2->show();
      signalLabel2->setText(QString::number(signal-256) + " dBm");

      noiseBar->hide();
      noiseLabel2->show();
      noiseLabel2->setText(QString::number(noise-256) + " dBm");
    }
  }
}
