/***************************************************************************
                          extendedwidget.cpp  -  description
                             -------------------
    begin                : Don Nov 14 2002
    copyright            : (C) 2002 by Matthias Reif
    email                : matthias.reif@informatik.tu-chemnitz.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 #include <iostream.h>
#include "extendedwidget.h"

ExtendedWidget::ExtendedWidget(WCard *pCard, QWidget *parent, const char *name ) : QWidget(parent,name)
{

  card = pCard;

  notString = new QString("<" + i18n ("not a wlan-device") + ">");

  bitRates = (BitRates*)malloc(sizeof(BitRates));

  timer = new QTimer(this);
  
  protocolLabel1 = new QLabel(this);
  protocolLabel1->setText(i18n ("Protocol") + ":");
  protocolLabel1->resize(100, 20);
  protocolLabel1->move(STARTX, STARTY + STEPY*0);
  protocolLabel1->show();

  protocolLabel2 = new QLabel(this);
  protocolLabel2->resize(150, 20);
  protocolLabel2->move(STARTX2, STARTY + STEPY*0);
  protocolLabel2->show();

  bitRateLabel1 = new QLabel(this);
  bitRateLabel1->setText(i18n ("Bit-Rate") + ":");
  bitRateLabel1->resize(100, 20);
  bitRateLabel1->move(STARTX, STARTY + STEPY*1);
  bitRateLabel1->show();

  bitRateLabel2 = new QLabel(this);
  bitRateLabel2->resize(150, 20);
  bitRateLabel2->move(STARTX2, STARTY + STEPY*1);
  bitRateLabel2->setText(*notString);
  bitRateLabel2->hide();

  bitRateCombo = new KComboBox(this);
  bitRateCombo->resize(100, 20);
  bitRateCombo->move(STARTX2, STARTY + STEPY*1);
  bitRateCombo->show();
  
  freqLabel1 = new QLabel(this);
  freqLabel1->setText(i18n ("Frequency") + ":");
  freqLabel1->resize(100, 20);
  freqLabel1->move(STARTX, STARTY + STEPY*2);
  freqLabel1->show();

  freqLabel2 = new QLabel(this);
  freqLabel2->resize(150, 20);  
  freqLabel2->move(STARTX2, STARTY + STEPY*2);
  freqLabel2->show();

  channelLabel1 = new QLabel(this);
  channelLabel1->setText(i18n ("Channel") + ":");
  channelLabel1->resize(100, 20);
  channelLabel1->move(STARTX, STARTY + STEPY*3);
  channelLabel1->show();

  channelLabel2 = new QLabel(this);
  channelLabel2->resize(150, 20);
  channelLabel2->move(STARTX2, STARTY + STEPY*3);
  channelLabel2->show();

  tpwrLabel1 = new QLabel(this);
  tpwrLabel1->setText(i18n ("Tx-Power") + ":");
  tpwrLabel1->resize(100, 20);
  tpwrLabel1->move(STARTX, STARTY + STEPY*4);
  tpwrLabel1->show();

  tpwrLabel2 = new QLabel(this);
  tpwrLabel2->resize(150, 20);
  tpwrLabel2->move(STARTX2, STARTY + STEPY*4);
  tpwrLabel2->show();

  sensLabel1 = new QLabel(this);
  sensLabel1->setText(i18n ("Sensitivity") + ":");
  sensLabel1->resize(100, 20);
  sensLabel1->move(STARTX, STARTY + STEPY*5);
  sensLabel1->show();

  sensLabel2 = new QLabel(this);
  sensLabel2->move(STARTX2, STARTY + STEPY*5);
  sensLabel2->resize(150, 20);
  sensLabel2->setText(*notString);
  sensLabel2->hide();

  sensCombo = new KComboBox(this);
  sensCombo->resize(100, 20);
  sensCombo->move(STARTX2, STARTY + STEPY*5);
  sensCombo->show();

  essidLabel = new QLabel(this);
  essidLabel->setText(i18n ("ESSID") + ":");
  essidLabel->resize(100, 20);
  essidLabel->move(STARTX, STARTY + STEPY*6);
  essidLabel->show();

  essidEdit = new KLineEdit(this);
  essidEdit->resize(100, 20);
  essidEdit->move(STARTX2, STARTY + STEPY*6);
  essidEdit->show();

  connect(bitRateCombo, SIGNAL(activated(int)), this, SLOT(slotBitRateChanged(int)));
  connect(sensCombo, SIGNAL(activated(int)), this, SLOT(slotSensChanged(int)));
  connect(essidEdit, SIGNAL(returnPressed()), this, SLOT(slotESSIDChanged()));
  
  connect(timer, SIGNAL(timeout()), this, SLOT(slotFreqChannel()));
  timer->start(1, false);
    
}

ExtendedWidget::~ExtendedWidget()
{
}

/** No descriptions */
void ExtendedWidget::slotAct()
{
  if(isVisible())
  {
    char* protocol = (char*)malloc(20);

    int sens, sensRange;

    if( card->getProtocol(protocol) >= 0 )
    {
      protocolLabel2->setText(protocol);

      tpwrLabel2->setText(QString::number(card->getTransmitPower()) + " dBm");

      int index;
      card->getBitRate(bitRates, index);
      bitRateCombo->setCurrentItem(index);
      
      int sens, sensRange;
      card->getSensitivity(sens, sensRange);
      sensCombo->setCurrentItem(sens-1);

    }else{
      protocolLabel2->setText(*notString);

      tpwrLabel2->setText(*notString);
      sensLabel2->setText(*notString);
      
    }
  }
}

/** No descriptions */
void ExtendedWidget::slotFreqChannel()
{
  if(isVisible())
  {
    if (card->isWave() )
    {
      int numFreq;
      double freq;
      int numChannel;
      int channel;
      char* eh = (char*)malloc(5);      
      if( card->getChannel(numFreq, freq, eh, numChannel, channel) < 0)
      {
        freqLabel2->setText(i18n ("error"));
        channelLabel2->setText(i18n ("error"));
      }else{
        freqLabel2->setText(QString::number(freq) + " " + eh);
        channelLabel2->setText(QString::number(channel) + " " + i18n ("of") + " " + QString::number(numChannel));
      }
      
    }else{
      freqLabel2->setText(*notString);
      channelLabel2->setText(*notString);
    }
  }      
      
}
/** No descriptions */
void ExtendedWidget::slotBitRates()
{
    if ( card->isWave() )
    {
      bitRateLabel2->hide();
      bitRateCombo->show();
      
      int index;
      card->getBitRate(bitRates, index);
      bitRateCombo->clear();
      for( int i=0; i < bitRates->num; i++)
      {
        bitRateCombo->insertItem(QString::number(bitRates->bitRate[i].value) + " " + bitRates->bitRate[i].unit, i);
      }
      bitRateCombo->setCurrentItem(index);


      sensLabel2->hide();
      sensCombo->show();

      int sens, sensRange;
      card->getSensitivity(sens, sensRange);

      sensCombo->clear();
      for(int i=0; i < sensRange; i++)
      {
        sensCombo->insertItem(QString::number(i+1) + " dBm", i);
      }
      sensCombo->setCurrentItem(sens-1);


      char* essid = (char*)malloc(IW_ESSID_MAX_SIZE+1);
      card->getESSID(essid);
      essidEdit->setText(essid);

    }else{
      bitRateCombo->hide();
      bitRateLabel2->show();
      sensLabel2->show();
      sensCombo->hide();      
    }
}

/** No descriptions */
void ExtendedWidget::slotBitRateChanged(int index)
{
  card->setBitRate(index);
  slotBitRates();
}
/** No descriptions */
void ExtendedWidget::slotSensChanged(int index)
{
   card->setSensitivity(index+1);
   slotBitRates();
}
/** No descriptions */
void ExtendedWidget::slotESSIDChanged()
{
  char *essid = (char*)malloc(strlen(essidEdit->text().data())+2);
  strcpy(essid, essidEdit->text().data() + '\0');
  card->setESSID(essid);
  slotBitRates();  
}
