/***************************************************************************
                          ktray.cpp  -  description
                             -------------------
    begin                : Die Nov 12 2002
    copyright            : (C) 2002 by Matthias Reif
    email                : matthias.reif@informatik.tu-chemnitz.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <klocale.h>
#include "ktray.h"

KTray::KTray(QWidget* parent, const char *name) : KSystemTray(parent, name)
{
  basicTimer = new QTimer(this);
  options = new Options();
  card = new WCard(options->getDevice()->data());

/*  
  QString optStr;
  confFile = new QFile("/home/remat/.kwavecontrol/kwavecontrol.conf");
  confFile->open(IO_ReadOnly);
  while(confFile->readLine(optStr, 100)>=0)
  {
     QString opt = optStr.section('=', 0, 0);
     QString val = optStr.section('=', 1, 1);

     if( opt == "device" )
      device = val;
  }
  */

  for(int i=0; i<=9; i++)
  {
    QString s;
    s = *(options->getKwcDir()) + "kwaveq" + QString::number(i) + ".png";
    icons[i].load(s);
  }
  icons[10].load(*(options->getKwcDir()) + "kwaveqx.png");
  
  widget = new WWidget(card, options);
  widget->resize(280, 350);
  widget->hide();

  popup = contextMenu();

  openAction = new QAction(this, "open");
  openAction->setText(i18n ("Open") );
  openAction->setIconSet(QPixmap(*(options->getKwcDir()) + "open.png"));
  openAction->addTo(popup);
  connect(openAction, SIGNAL(activated()), this, SLOT(slotOpen()));

  closeAction = new QAction(this, "close");
  closeAction->setText(i18n ("Close") );
  closeAction->setIconSet(QPixmap(*(options->getKwcDir()) + "close.png"));
  closeAction->addTo(popup);
  connect(closeAction, SIGNAL(activated()), this, SLOT(slotClose()));

  connect(options, SIGNAL(pollingChanged(int)), this, SLOT(slotPolling(int)));  

  connect(basicTimer, SIGNAL(timeout()), this, SLOT(slotAct()));
  basicTimer->start(500, false);

  state = 0;
  setState(state);
}

KTray::~KTray()
{
}

/** No descriptions */
void KTray::setState(int value)
{
  state = value;
  setPixmap(icons[value]);
}

/** No descriptions */
void KTray::slotAct()
{

  if( card->isWave() )
  {
    card->getLinkQual(link, avg, signal, noise);

    if(link < 0)
      link = 0;
    if(link > 100)
      link =100;
    
    QToolTip::add(this, QString::number(link) + '%');
  
    int newState = link / 10;
    if( link < 10 && link > 0)
      newState = 1;
    if( link >= 100 )
      newState = 9;
  
    if( newState != state )
    {
      setState(newState);
      state = newState;
    }

    
  }else{
    QToolTip::add(this, *options->getNotString());
    setState(10);
  }
  
  if(widget->isVisible())
  {
    widget->act(link, avg, signal, noise);
  }
    
}

void KTray::slotOpen()
{
  widget->show();
  widget->setFocus();
}
void KTray::slotClose()
{
  widget->hide();
}

/** No descriptions */
void KTray::mousePressEvent ( QMouseEvent *ev )
{
  if( ev->button() == LeftButton )
  {
    if(widget->isVisible())
    {
      widget->hide();      
    }else{
      QPoint p;
      p.setX(ev->globalX() - (widget->width() / 2) );
      p.setY(ev->globalY() - (widget->height() + 50) );
      widget->move(p);
      slotOpen();
    }
  }
  
  if( ev->button() == RightButton )
    popup->exec(ev->globalPos());    
}
/** No descriptions */
void KTray::slotPolling(int value)
{
  basicTimer->stop();
  basicTimer->start(value*100, false);
}
